@@ -1,7 +1,7 @@
 """
 MIT License
 
-Copyright (c) 2023-present Qvco, Konn
+Copyright (c) 2023-present qvco
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -36,12 +36,12 @@ def get_user_activities(self, access_token: str = None, **params) -> ActivitiesR
         - number: int - (optional)
 
     """
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"https://{Configs.YAY_STAGING_HOST_2}/api/user_activities",
         params=params,
         data_type=ActivitiesResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
@@ -57,12 +57,12 @@ def get_user_merged_activities(
         - number: int - (optional)
 
     """
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"https://{Configs.YAY_STAGING_HOST_2}/api/v2/user_activities",
         params=params,
         data_type=ActivitiesResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
@@ -71,10 +71,10 @@ def received_notification(
     self, pid: str, type: str, opened_at: int = None, access_token: str = None
 ):
     # TODO: opened_atはnullalbeか確認する
-    self._check_authorization(access_token)
     return self._make_request(
         "POST",
         endpoint=f"{Endpoints.BASE_API_URL}/api/received_push_notifications",
         payload={"pid": pid, "type": type, "opened_at": opened_at},
+        auth_required=True,
         access_token=access_token,
     )
