@@ -1,7 +1,7 @@
 """
 MIT License
 
-Copyright (c) 2023-present Qvco, Konn
+Copyright (c) 2023-present qvco
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -22,7 +22,7 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 """
 
-from typing import List
+from __future__ import annotations
 
 from ..config import Endpoints
 from ..models import ChatRoom, GifImageCategory, Message, StickerPack
@@ -40,27 +40,27 @@ from ..responses import (
 )
 
 
-def accept_chat_requests(self, chat_room_ids: List[int], access_token: str = None):
-    self._check_authorization(access_token)
+def accept_chat_requests(self, chat_room_ids: list[int], access_token: str = None):
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.CHAT_ROOMS_V1}/accept_chat_request",
-        payload={"chat_room_ids": chat_room_ids},
+        payload={"chat_room_ids[]": chat_room_ids},
+        auth_required=True,
         access_token=access_token,
     )
-    self.logger.info("Accepted chat requests")
+    self.logger.info("Accepted chat requests.")
     return response
 
 
 def check_unread_status(
     self, from_time: int, access_token: str = None
 ) -> UnreadStatusResponse:
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.CHAT_ROOMS_V1}/unread_status",
         params={"from_time": from_time},
         data_type=UnreadStatusResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
@@ -68,12 +68,11 @@ def check_unread_status(
 def create_group_chat(
     self,
     name: str,
-    with_user_ids: List[int],
+    with_user_ids: list[int],
     icon_filename: str = None,
     background_filename: str = None,
     access_token: str = None,
 ) -> CreateChatRoomResponse:
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.CHAT_ROOMS_V3}/new",
@@ -84,6 +83,7 @@ def create_group_chat(
             "background_filename": background_filename,
         },
         data_type=CreateChatRoomResponse,
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info(f"Group chat '{name}' has been created.")
@@ -97,7 +97,6 @@ def create_private_chat(
     hima_chat: bool = False,
     access_token: str = None,
 ) -> CreateChatRoomResponse:
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.CHAT_ROOMS_V1}/new",
@@ -107,6 +106,7 @@ def create_private_chat(
             "hima_chat": hima_chat,
         },
         data_type=CreateChatRoomResponse,
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info(f"Created a private chatroom with '{with_user_id}'.")
@@ -114,10 +114,10 @@ def create_private_chat(
 
 
 def delete_background(self, room_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "DELETE",
         endpoint=f"{Endpoints.CHAT_ROOMS_V2}/{room_id}/background",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Background image of the chatroom has been deleted.")
@@ -125,10 +125,10 @@ def delete_background(self, room_id: int, access_token: str = None):
 
 
 def delete_message(self, room_id: int, message_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "DELETE",
         endpoint=f"{Endpoints.CHAT_ROOMS_V1}/{room_id}/messages/{message_id}/delete",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Message has been deleted.")
@@ -143,7 +143,6 @@ def edit_chat_room(
     background_filename: str = None,
     access_token: str = None,
 ):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.CHAT_ROOMS_V1}/{chat_room_id}/edit",
@@ -152,6 +151,7 @@ def edit_chat_room(
             "icon_filename": icon_filename,
             "background_filename": background_filename,
         },
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Chatroom has been updated.")
@@ -166,7 +166,6 @@ def get_chatable_users(
     order_by: str = None,
     access_token: str = None,
 ) -> FollowUsersResponse:
-    self._check_authorization(access_token)
     return self._make_request(
         "POST",
         endpoint=f"{Endpoints.USERS_V1}/followings/chatable",
@@ -176,16 +175,17 @@ def get_chatable_users(
             "order_by": order_by,
         },
         data_type=FollowUsersResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
 
-def get_gifs_data(self, access_token: str = None) -> List[GifImageCategory]:
-    self._check_authorization(access_token)
+def get_gifs_data(self, access_token: str = None) -> list[GifImageCategory]:
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.HIDDEN_V1}/chats",
         data_type=GifsDataResponse,
+        auth_required=True,
         access_token=access_token,
     ).gif_categories
 
@@ -202,12 +202,12 @@ def get_hidden_chat_rooms(
         - number: int - (optional)
 
     """
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.HIDDEN_V1}/chats",
         params=params,
         data_type=ChatRoomsResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
@@ -215,7 +215,6 @@ def get_hidden_chat_rooms(
 def get_main_chat_rooms(
     self, from_timestamp: int = None, access_token: str = None
 ) -> ChatRoomsResponse:
-    self._check_authorization(access_token)
     params = {}
     if from_timestamp:
         params["from_timestamp"] = from_timestamp
@@ -224,13 +223,14 @@ def get_main_chat_rooms(
         endpoint=f"{Endpoints.CHAT_ROOMS_V1}/main_list",
         params=params,
         data_type=ChatRoomsResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
 
 def get_messages(
     self, chat_room_id: int, access_token: str = None, **params
-) -> List[Message]:
+) -> list[Message]:
     """
 
     Parameters:
@@ -239,12 +239,12 @@ def get_messages(
         - to_message_id: int - (optional)
 
     """
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.CHAT_ROOMS_V2}/{chat_room_id}/messages",
         params=params,
         data_type=MessagesResponse,
+        auth_required=True,
         access_token=access_token,
     ).messages
 
@@ -262,51 +262,52 @@ def get_request_chat_rooms(
         - access_token: str (optional)
 
     """
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.CHAT_ROOMS_V1}/request_list",
         params=params,
         data_type=ChatRoomsResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
 
 def get_chat_room(self, chat_room_id: int, access_token: str = None) -> ChatRoom:
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.CHAT_ROOMS_V2}/{chat_room_id}",
         data_type=ChatRoomResponse,
+        auth_required=True,
         access_token=access_token,
     ).chat
 
 
-def get_sticker_packs(self, access_token: str = None) -> List[StickerPack]:
+def get_sticker_packs(self, access_token: str = None) -> list[StickerPack]:
     return self._make_request(
         "GET",
         endpoint=Endpoints.STICKER_PACKS_V2,
         data_type=StickerPacksResponse,
+        auth_required=True,
         access_token=access_token,
     ).sticker_packs
 
 
 def get_total_chat_requests(self, access_token: str = None) -> int:
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.CHAT_ROOMS_V1}/total_chat_request",
         data_type=TotalChatRequestResponse,
+        auth_required=True,
         access_token=access_token,
     ).total
 
 
 def hide_chat(self, chat_room_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.HIDDEN_V1}/chats",
         payload={"chat_room_id": chat_room_id},
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info(f"Chatroom '{chat_room_id}' has been hidden.")
@@ -314,13 +315,13 @@ def hide_chat(self, chat_room_id: int, access_token: str = None):
 
 
 def invite_to_chat(
-    self, chat_room_id: int, user_ids: List[int], access_token: str = None
+    self, chat_room_id: int, user_ids: list[int], access_token: str = None
 ):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.CHAT_ROOMS_V2}/{chat_room_id}/invite",
         payload={"with_user_ids": user_ids},
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Invited users to the chatroom.")
@@ -328,13 +329,13 @@ def invite_to_chat(
 
 
 def kick_users_from_chat(
-    self, chat_room_id: int, user_ids: List[int], access_token: str = None
+    self, chat_room_id: int, user_ids: list[int], access_token: str = None
 ):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.CHAT_ROOMS_V2}/{chat_room_id}/kick",
         payload={"with_user_ids[]": user_ids},
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info(f"Users have been kicked from the chatroom.")
@@ -342,10 +343,10 @@ def kick_users_from_chat(
 
 
 def pin_chat(self, room_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.CHAT_ROOMS_V1}/{room_id}/pinned",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Pinned the chatroom.")
@@ -353,10 +354,10 @@ def pin_chat(self, room_id: int, access_token: str = None):
 
 
 def read_message(self, chat_room_id: int, message_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.CHAT_ROOMS_V2}/{chat_room_id}/messages/{message_id}/read",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Message has been read.")
@@ -366,7 +367,6 @@ def read_message(self, chat_room_id: int, message_id: int, access_token: str = N
 def refresh_chat_rooms(
     self, from_time: int = None, access_token: str = None
 ) -> ChatRoomsResponse:
-    self._check_authorization(access_token)
     params = {}
     if from_time:
         params["from_time"] = from_time
@@ -375,17 +375,18 @@ def refresh_chat_rooms(
         endpoint=f"{Endpoints.CHAT_ROOMS_V2}/update",
         params=params,
         data_type=ChatRoomsResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
 
-def remove_chat_rooms(self, chat_room_ids: List[int], access_token: str = None):
-    self._check_authorization(access_token)
+def remove_chat_rooms(self, chat_room_ids: list[int], access_token: str = None):
     chat_room_ids = [chat_room_ids] if isinstance(chat_room_ids, int) else chat_room_ids
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.CHAT_ROOMS_V1}/mass_destroy",
         payload={"chat_room_ids": chat_room_ids},
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info(f"Chatrooms have been removed.")
@@ -404,7 +405,6 @@ def report_chat_room(
     screenshot_4_filename: str = None,
     access_token: str = None,
 ):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.CHAT_ROOMS_V3}/{chat_room_id}/report",
@@ -418,6 +418,7 @@ def report_chat_room(
             "screenshot_3_filename": screenshot_3_filename,
             "screenshot_4_filename": screenshot_4_filename,
         },
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info(f"Chatroom '{chat_room_id}' has been reported.")
@@ -425,35 +426,14 @@ def report_chat_room(
 
 
 def send_message(
-    self,
-    chat_room_id: int,
-    message_type: str,
-    call_type: str = None,
-    text: str = None,
-    font_size: int = None,
-    gif_image_id: int = None,
-    attachment_file_name: str = None,
-    sticker_pack_id: int = None,
-    sticker_id: int = None,
-    video_file_name: str = None,
-    access_token: str = None,
+    self, chat_room_id: int, access_token: str = None, **params
 ) -> MessageResponse:
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.CHAT_ROOMS_V3}/{chat_room_id}/messages/new",
-        payload={
-            "message_type": message_type,
-            "call_type": call_type,
-            "text": text,
-            "font_size": font_size,
-            "gif_image_id": gif_image_id,
-            "attachment_file_name": attachment_file_name,
-            "sticker_pack_id": sticker_pack_id,
-            "sticker_id": sticker_id,
-            "video_file_name": video_file_name,
-        },
+        payload=params,
         data_type=MessageResponse,
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Your message has been sent.")
@@ -461,23 +441,23 @@ def send_message(
 
 
 def unhide_chat(self, chat_room_ids: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "DELETE",
         endpoint=f"{Endpoints.HIDDEN_V1}/chats",
         params={"chat_room_ids": chat_room_ids},
+        auth_required=True,
         access_token=access_token,
     )
-    self.logger.info("Unhid the chatrooms")
+    self.logger.info("Unhid the chatrooms.")
     return response
 
 
 def unpin_chat(self, chat_room_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "DELETE",
         endpoint=f"{Endpoints.CHAT_ROOMS_V1}/{chat_room_id}/pinned",
+        auth_required=True,
         access_token=access_token,
     )
-    self.logger.info("Unpinned the chatroom")
+    self.logger.info("Unpinned the chatroom.")
     return response
