@@ -1,7 +1,7 @@
 """
 MIT License
 
-Copyright (c) 2023-present Qvco, Konn
+Copyright (c) 2023-present qvco
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -22,8 +22,9 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 """
 
+from __future__ import annotations
+
 from datetime import datetime
-from typing import List
 
 from ..config import Endpoints
 from ..models import CreateGroupQuota
@@ -43,10 +44,10 @@ from ..responses import (
 
 
 def accept_moderator_offer(self, group_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "PUT",
         endpoint=f"{Endpoints.GROUPS_V1}/{group_id}/deputize",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Accepted the group moderator offer.")
@@ -54,10 +55,10 @@ def accept_moderator_offer(self, group_id: int, access_token: str = None):
 
 
 def accept_ownership_offer(self, group_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "PUT",
         endpoint=f"{Endpoints.GROUPS_V1}/{group_id}/transfer",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Accepted the group ownership offer.")
@@ -67,10 +68,10 @@ def accept_ownership_offer(self, group_id: int, access_token: str = None):
 def accept_group_join_request(
     self, group_id: int, user_id: int, access_token: str = None
 ):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.GROUPS_V1}/{group_id}/accept/{user_id}",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Accepted the group join request.")
@@ -78,29 +79,29 @@ def accept_group_join_request(
 
 
 def add_related_groups(
-    self, group_id: int, related_group_id: List[int], access_token: str = None
+    self, group_id: int, related_group_id: list[int], access_token: str = None
 ):
     """
 
     関連サークルを追加する
 
     """
-    self._check_authorization(access_token)
     response = self._make_request(
         "PUT",
         endpoint=f"{Endpoints.GROUPS_V1}/{group_id}/related",
         params={"related_group_id[]": related_group_id},
+        auth_required=True,
         access_token=access_token,
     )
-    self.logger.info("Group has been added to the related groups")
+    self.logger.info("Group has been added to the related groups.")
     return response
 
 
 def ban_group_user(self, group_id: int, user_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.GROUPS_V1}/{group_id}/ban/{user_id}",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info(f"User '{user_id}' has been banned from the group.")
@@ -110,7 +111,6 @@ def ban_group_user(self, group_id: int, user_id: int, access_token: str = None):
 def check_unread_status(
     self, from_time: int = None, access_token: str = None
 ) -> UnreadStatusResponse:
-    self._check_authorization(access_token)
     params = {}
     if from_time:
         params["from_time"] = from_time
@@ -119,6 +119,7 @@ def check_unread_status(
         endpoint=f"{Endpoints.GROUPS_V1}/unread_status",
         params=params,
         data_type=UnreadStatusResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
@@ -147,7 +148,6 @@ def create_group(
     guidelines: str = None,
     access_token: str = None,
 ) -> CreateGroupResponse:
-    self._check_authorization(access_token)
     timestamp = int(datetime.now().timestamp())
     response = self._make_request(
         "POST",
@@ -179,6 +179,7 @@ def create_group(
             "guidelines": guidelines,
         },
         data_type=CreateGroupResponse,
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Group has been created.")
@@ -186,11 +187,11 @@ def create_group(
 
 
 def create_pin_group(self, group_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.PINNED_V1}/groups",
         payload={"id": group_id},
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Pinned the group.")
@@ -198,10 +199,10 @@ def create_pin_group(self, group_id: int, access_token: str = None):
 
 
 def decline_moderator_offer(self, group_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "DELETE",
         endpoint=f"{Endpoints.GROUPS_V1}/{group_id}/deputize",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Declined the moderator offer.")
@@ -209,10 +210,10 @@ def decline_moderator_offer(self, group_id: int, access_token: str = None):
 
 
 def decline_ownership_offer(self, group_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "DELETE",
         endpoint=f"{Endpoints.GROUPS_V1}/{group_id}/transfer",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Declined the ownership offer.")
@@ -222,10 +223,10 @@ def decline_ownership_offer(self, group_id: int, access_token: str = None):
 def decline_group_join_request(
     self, group_id: int, user_id: int, access_token: str = None
 ):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.GROUPS_V1}/{group_id}/decline/{user_id}",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Declined the group join request.")
@@ -233,10 +234,10 @@ def decline_group_join_request(
 
 
 def delete_pin_group(self, group_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "DELETE",
         endpoint=f"{Endpoints.PINNED_V1}/groups/{group_id}",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Unpinned the group.")
@@ -246,7 +247,6 @@ def delete_pin_group(self, group_id: int, access_token: str = None):
 def get_banned_group_members(
     self, group_id: int, page: int = None, access_token: str = None
 ) -> UsersResponse:
-    self._check_authorization(access_token)
     params = {}
     if page:
         params["page"] = page
@@ -254,6 +254,7 @@ def get_banned_group_members(
         "GET",
         endpoint=f"{Endpoints.GROUPS_V1}/{group_id}/ban_list",
         params=params,
+        auth_required=True,
         access_token=access_token,
     )
 
@@ -339,12 +340,12 @@ def get_invitable_users(
     )
 
 
-def get_joined_statuses(self, ids: List[int], access_token: str = None) -> dict:
-    self._check_authorization(access_token)
+def get_joined_statuses(self, ids: list[int], access_token: str = None) -> dict:
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.GROUPS_V1}/joined_statuses",
         params={"ids[]": ids},
+        auth_required=True,
         access_token=access_token,
     )
 
@@ -388,7 +389,6 @@ def get_group_members(
 def get_my_groups(
     self, from_timestamp: None, access_token: str = None
 ) -> GroupsResponse:
-    self._check_authorization(access_token)
     params = {}
     if from_timestamp:
         params["from_timestamp"] = from_timestamp
@@ -397,6 +397,7 @@ def get_my_groups(
         endpoint=f"{Endpoints.GROUPS_V2}/mine",
         params=params,
         data_type=GroupsResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
@@ -465,13 +466,13 @@ def get_user_groups(self, access_token: str = None, **params) -> GroupsResponse:
 
 
 def invite_users_to_group(
-    self, group_id: int, user_ids: List[int], access_token: str = None
+    self, group_id: int, user_ids: list[int], access_token: str = None
 ):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.GROUPS_V1}/{group_id}/invite",
         payload={"user_ids[]": user_ids},
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Invited users to the group.")
@@ -479,10 +480,10 @@ def invite_users_to_group(
 
 
 def join_group(self, group_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.GROUPS_V1}/{group_id}/join",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("You are now one of the members of the group.")
@@ -490,10 +491,10 @@ def join_group(self, group_id: int, access_token: str = None):
 
 
 def leave_group(self, group_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "DELETE",
         endpoint=f"{Endpoints.GROUPS_V1}/{group_id}/leave",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Left the group.")
@@ -503,11 +504,11 @@ def leave_group(self, group_id: int, access_token: str = None):
 def post_gruop_social_shared(
     self, group_id: int, sns_name: str, access_token: str = None
 ):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.GROUPS_V2}/{group_id}/social_shared",
         params={"sns_name": sns_name},
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Group social shared has been posted.")
@@ -515,10 +516,10 @@ def post_gruop_social_shared(
 
 
 def remove_group_cover(self, group_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.GROUPS_V1}/{group_id}/remove_cover",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Group cover image has been removed.")
@@ -526,10 +527,10 @@ def remove_group_cover(self, group_id: int, access_token: str = None):
 
 
 def remove_moderator(self, group_id: int, user_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.GROUPS_V1}/{group_id}/fire/{user_id}",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info(f"Group moderator '{user_id}' has been removed.")
@@ -537,13 +538,13 @@ def remove_moderator(self, group_id: int, user_id: int, access_token: str = None
 
 
 def remove_related_groups(
-    self, group_id: int, related_group_ids: List[int], access_token: str = None
+    self, group_id: int, related_group_ids: list[int], access_token: str = None
 ):
-    self._check_authorization(access_token)
     response = self._make_request(
         "DELETE",
         endpoint=f"{Endpoints.GROUPS_V1}/{group_id}/related",
         params={"related_group_id[]": related_group_ids},
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Related groups have been removed.")
@@ -562,7 +563,6 @@ def report_group(
     screenshot_4_filename: str = None,
     access_token: str = None,
 ):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.GROUPS_V3}/{group_id}/report",
@@ -575,6 +575,7 @@ def report_group(
             "screenshot_3_filename": screenshot_3_filename,
             "screenshot_4_filename": screenshot_4_filename,
         },
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Group has been reported.")
@@ -582,9 +583,8 @@ def report_group(
 
 
 def send_moderator_offers(
-    self, group_id: int, user_ids: List[int], access_token: str = None
+    self, group_id: int, user_ids: list[int], access_token: str = None
 ):
-    self._check_authorization(access_token)
     timestamp = int(datetime.now().timestamp())
     response = self._make_request(
         "POST",
@@ -596,6 +596,7 @@ def send_moderator_offers(
             "timestamp": timestamp,
             "signed_info": self.generate_signed_info(self.device_uuid, timestamp),
         },
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Offered users to become a group moderator.")
@@ -603,7 +604,6 @@ def send_moderator_offers(
 
 
 def send_ownership_offer(self, group_id: int, user_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     timestamp = int(datetime.now().timestamp())
     response = self._make_request(
         "POST",
@@ -615,6 +615,7 @@ def send_ownership_offer(self, group_id: int, user_id: int, access_token: str =
             "timestamp": timestamp,
             "signed_info": self.generate_signed_info(self.device_uuid, timestamp),
         },
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Offered user to become a group owner.")
@@ -622,11 +623,11 @@ def send_ownership_offer(self, group_id: int, user_id: int, access_token: str =
 
 
 def set_group_title(self, group_id: int, title: str, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.GROUPS_V1}/{group_id}/set_title",
         payload={"title": title},
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Group tittle has been set.")
@@ -634,21 +635,21 @@ def set_group_title(self, group_id: int, title: str, access_token: str = None):
 
 
 def take_over_group_ownership(self, group_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.GROUPS_V1}/{group_id}/take_over",
+        auth_required=True,
         access_token=access_token,
     )
-    self.logger.info(f"Took over the group ownership of {group_id}")
+    self.logger.info(f"Took over the group ownership of {group_id}.")
     return response
 
 
 def unban_group_member(self, group_id: int, user_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.GROUPS_V1}/{group_id}/unban/{user_id}",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("User has been banned from the group.")
@@ -680,7 +681,6 @@ def update_group(
     guidelines: str = None,
     access_token: str = None,
 ) -> Group:
-    self._check_authorization(access_token)
     timestamp = int(datetime.now().timestamp())
     response = self._make_request(
         "POST",
@@ -712,6 +712,7 @@ def update_group(
             "guidelines": guidelines,
         },
         data_type=GroupResponse,
+        auth_required=True,
         access_token=access_token,
     ).group
     self.logger.info("Group details have been updated.")
@@ -721,25 +722,25 @@ def update_group(
 def withdraw_moderator_offer(
     self, group_id: int, user_id: int, access_token: str = None
 ):
-    self._check_authorization(access_token)
     response = self._make_request(
         "PUT",
         endpoint=f"{Endpoints.GROUPS_V1}/{group_id}/deputize/{user_id}/withdraw",
+        auth_required=True,
         access_token=access_token,
     )
-    self.logger.info("Group moderator offer has been withdrawn")
+    self.logger.info("Group moderator offer has been withdrawn.")
     return response
 
 
 def withdraw_ownership_offer(
     self, group_id: int, user_id: int, access_token: str = None
 ):
-    self._check_authorization(access_token)
     response = self._make_request(
         "PUT",
         endpoint=f"{Endpoints.GROUPS_V1}/{group_id}/transfer/withdraw",
         payload={"user_id": user_id},
+        auth_required=True,
         access_token=access_token,
     )
-    self.logger.info("Group ownership offer has been withdrawn")
+    self.logger.info("Group ownership offer has been withdrawn.")
     return response
