@@ -1,7 +1,7 @@
 """
 MIT License
 
-Copyright (c) 2023-present Qvco, Konn
+Copyright (c) 2023-present qvco
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -23,12 +23,18 @@ SOFTWARE.
 """
 
 import hashlib
-from datetime import datetime
+
 from cryptography.fernet import Fernet
+from datetime import datetime
 
 from ..config import Endpoints
-from ..errors import AuthenticationError, ForbiddenError
-from ..responses import LoginUserResponse, LoginUpdateResponse, TokenResponse
+from ..errors import ForbiddenError
+from ..responses import (
+    LoginUserResponse,
+    LoginUpdateResponse,
+    RegisterDeviceTokenResponse,
+    TokenResponse,
+)
 from ..utils import Colors, console_print
 
 
@@ -69,7 +75,7 @@ def change_password(
         data_type=LoginUpdateResponse,
         access_token=access_token,
     )
-    self.logger.info("Your password has been changed..")
+    self.logger.info("Your password has been updated.")
     return response
 
 
@@ -91,44 +97,43 @@ def get_token(
             "refresh_token": refresh_token,
         },
         data_type=TokenResponse,
+        bypass_delay=True,
         access_token=access_token,
     )
 
 
-def is_valid_token(self, access_token: str):
-    headers = self.session.headers
-    headers.setdefault("Authorization", f"Bearer {access_token}")
-    try:
-        self.get_web_socket_token(headers)
-        return True
-    except AuthenticationError:
-        return False
-
-
-def login_with_email(
+def login_flow(
     self, email: str, password: str, secret_key: str = None
 ) -> LoginUserResponse:
-    """
+    self.logger.debug("Starting LOGIN FLOW!")
 
-    メールアドレスでログインします
-
-    ローカルストレージのトークンの暗号化を利用するには、`Client` クラスの `encrypt_cookie` 引数を`True` に設定してください。
-
-    """
     if self.save_cookie_file:
         loaded_cookies = self.load_cookies()
         hashed_email = hashlib.sha256(email.encode()).hexdigest()
 
-        if loaded_cookies is not None and loaded_cookies.get("email") == hashed_email:
+        if (
+            loaded_cookies is not None
+            and loaded_cookies.get("user", {}).get("email") == hashed_email
+        ):
             if not self.encrypt_cookie:
                 self.cookies = loaded_cookies
                 # email property is reassigned here because it's hashed
                 self.email = email
-                self.session.headers.setdefault(
-                    "Authorization", f"Bearer {self.access_token}"
+                self.session.headers.update(
+                    {
+                        "Authorization": "Bearer " + self.access_token,
+                        "X-Device-UUID": self.device_uuid,
+                    }
+                )
+                self.logger.info(f"Successfully logged in as '{self.user_id}'.")
+                return LoginUserResponse(
+                    {
+                        "access_token": self.access_token,
+                        "refresh_token": self.refresh_token,
+                        "user_id": self.user_id,
+                        "email": self.email,
+                    }
                 )
-                self.logger.info(f"Successfully logged in as '{self.user_id}'")
-                return LoginUserResponse(self.cookies)
 
             if secret_key is not None:
                 self.secret_key = secret_key
@@ -136,39 +141,41 @@ def login_with_email(
                 self.cookies = self.decrypt_cookies(self.fernet, loaded_cookies)
                 # email property is reassigned here because it's hashed
                 self.email = email
-                self.session.headers.setdefault(
-                    "Authorization", f"Bearer {self.access_token}"
+                self.session.headers.update(
+                    {
+                        "Authorization": "Bearer " + self.access_token,
+                        "X-Device-UUID": self.device_uuid,
+                    }
+                )
+                self.logger.info(f"Successfully logged in as '{self.user_id}'.")
+                return LoginUserResponse(
+                    {
+                        "access_token": self.access_token,
+                        "refresh_token": self.refresh_token,
+                        "user_id": self.user_id,
+                        "email": self.email,
+                    }
                 )
-                self.logger.info(f"Successfully logged in as '{self.user_id}'")
-                return LoginUserResponse(self.cookies)
 
             console_print(
-                f"{Colors.WARNING}Cookie データが見つかりました。「secret_key」を設定することにより、ログインレート制限を回避できます。{Colors.RESET}"
+                f"{Colors.WARNING}Cookie データが見つかりました。"
+                + f"「secret_key」を設定することにより、ログインレート制限を回避できます。{Colors.RESET}"
             )
 
-    response = self._make_request(
-        "POST",
-        endpoint=f"{Endpoints.USERS_V3}/login_with_email",
-        payload={
-            "api_key": self.api_key,
-            "email": email,
-            "password": password,
-            "uuid": self.uuid,
-        },
-        data_type=LoginUserResponse,
-    )
+    response = login_with_email(self, email, password)
 
-    if response.access_token is None:
-        raise ForbiddenError("Invalid email or password.")
-
-    self.session.headers.setdefault("Authorization", f"Bearer {response.access_token}")
-    self.logger.info(f"Successfully logged in as '{response.user_id}'")
+    self.session.headers["Authorization"] = "Bearer " + response.access_token
 
     self.cookies = {
-        "access_token": response.access_token,
-        "refresh_token": response.refresh_token,
-        "user_id": response.user_id,
-        "email": email,
+        "authentication": {
+            "access_token": response.access_token,
+            "refresh_token": response.refresh_token,
+        },
+        "user": {
+            "user_id": response.user_id,
+            "email": email,
+        },
+        "device": {"device_uuid": self.device_uuid},
     }
 
     if self.save_cookie_file:
@@ -190,13 +197,36 @@ def login_with_email(
     return response
 
 
+def login_with_email(self, email: str, password: str) -> LoginUserResponse:
+    """
+
+    メールアドレスでログインします
+
+    """
+    response = self._make_request(
+        "POST",
+        endpoint=f"{Endpoints.USERS_V3}/login_with_email",
+        payload={
+            "api_key": self.api_key,
+            "email": email,
+            "password": password,
+            "uuid": self.uuid,
+        },
+        data_type=LoginUserResponse,
+    )
+    if response.access_token is None:
+        raise ForbiddenError("Invalid email or password.")
+    self.logger.info(f"Successfully logged in as '{response.user_id}'.")
+    return response
+
+
 def logout(self, access_token: str = None):
     try:
-        self._check_authorization(access_token)
         response = self._make_request(
             "POST",
             endpoint=f"{Endpoints.USERS_V1}/logout",
             payload={"uuid": self.uuid},
+            auth_required=True,
             access_token=access_token,
         )
         self._cookies = {}
@@ -235,6 +265,41 @@ def restore_user(self, user_id: int, access_token: str = None) -> LoginUserRespo
     return response
 
 
+def register_device_token(
+    self,
+    device_token: str,
+    device_type: str,
+    os_version: str,
+    app_version: str,
+    screen_resolution: str,
+    screen_density: str,
+    device_model: str,
+    appsflyer_id: str,
+    advertising_id: str = None,
+    access_token: str = None,
+) -> RegisterDeviceTokenResponse:
+    response = self._make_request(
+        "POST",
+        endpoint=f"{Endpoints.USERS_V2}/device_tokens/new",
+        payload={
+            "device_token": device_token,
+            "device_type": device_type,
+            "uuid": self.uuid,
+            "os_version": os_version,
+            "app_version": app_version,
+            "screen_resolution": screen_resolution,
+            "screen_density": screen_density,
+            "device_model": device_model,
+            "appsflyer_id": appsflyer_id,
+            "advertising_id": advertising_id,
+        },
+        data_type=RegisterDeviceTokenResponse,
+        access_token=access_token,
+    )
+    self.logger.info("Device token has been registered.")
+    return response
+
+
 def revoke_tokens(self, access_token: str = None):
     response = self._make_request(
         "DELETE",
