@@ -1,7 +1,7 @@
 """
 MIT License
 
-Copyright (c) 2023-present Qvco, Konn
+Copyright (c) 2023-present qvco
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -22,11 +22,12 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 """
 
+from __future__ import annotations
+
 import os
 import httpx
 from io import BytesIO
 from PIL import Image
-from typing import List
 from urllib import parse
 
 from ..config import Endpoints, Configs
@@ -42,6 +43,7 @@ from ..responses import (
     ApplicationConfigResponse,
     BanWordsResponse,
     PopularWordsResponse,
+    PolicyAgreementsResponse,
 )
 from ..utils import is_valid_image_format, get_hashed_filename
 
@@ -63,9 +65,10 @@ def accept_policy_agreement(self, type: str, access_token: str = None):
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.USERS_V1}/policy_agreements/{type}",
+        bypass_delay=True,
         access_token=access_token,
     )
-    self.logger.info("Accepted to policy agreement.")
+    self.logger.info(f"Accepted to {type}.")
     return response
 
 
@@ -123,18 +126,20 @@ def get_email_verification_presigned_url(
             "intent": intent,
         },
         data_type=EmailVerificationPresignedUrlResponse,
+        bypass_delay=True,
         access_token=access_token,
     )
 
 
 def get_file_upload_presigned_urls(
-    self, file_names: List[str], access_token: str = None
-) -> List[PresignedUrl]:
+    self, file_names: list[str], access_token: str = None
+) -> list[PresignedUrl]:
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.BUCKETS_V1}/presigned_urls",
         params={"file_names[]": file_names},
         data_type=PresignedUrlsResponse,
+        bypass_delay=True,
         access_token=access_token,
     ).presigned_urls
 
@@ -151,6 +156,7 @@ def get_id_checker_presigned_url(
         endpoint=f"{Endpoints.ID_CHECK_V1}/{model}/{action}",
         params=params,
         data_type=IdCheckerPresignedUrlResponse,
+        bypass_delay=True,
         access_token=access_token,
     ).presigned_url
 
@@ -163,17 +169,29 @@ def get_old_file_upload_presigned_url(
         endpoint=f"{Endpoints.USERS_V1}/presigned_url",
         params={"video_file_name": video_file_name},
         data_type=PresignedUrlResponse,
+        bypass_delay=True,
         access_token=access_token,
     ).presigned_url
 
 
+def get_policy_agreements(self, access_token: str = None) -> PolicyAgreementsResponse:
+    return self._make_request(
+        "GET",
+        endpoint=f"{Endpoints.USERS_V1}/policy_agreements",
+        data_type=PolicyAgreementsResponse,
+        bypass_delay=True,
+        access_token=access_token,
+    )
+
+
 def get_web_socket_token(self, headers: dict = None, access_token: str = None) -> str:
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.USERS_V1}/ws_token",
         data_type=WebSocketTokenResponse,
         headers=headers,
+        auth_required=True,
+        bypass_delay=True,
         access_token=access_token,
     ).token
 
@@ -203,7 +221,7 @@ def verify_device(
     return response
 
 
-def upload_image(self, image_paths: List[str], image_type: str) -> List[Attachment]:
+def upload_image(self, image_paths: list[str], image_type: str) -> list[Attachment]:
     """
 
     画像をアップロードして、サーバー上のファイル名のリストを返します。
@@ -211,7 +229,7 @@ def upload_image(self, image_paths: List[str], image_type: str) -> List[Attachme
     Parameteres
     -----------
 
-        - image_path: List[str] - (required): 画像パスのリスト
+        - image_path: list[str] - (required): 画像パスのリスト
         - image_type: str - (required): 画像の種類
 
     Examples
@@ -269,7 +287,7 @@ def upload_image(self, image_paths: List[str], image_type: str) -> List[Attachme
             is_thumb=True,
         )
 
-        uuid = self.generate_uuid()[:8]
+        uuid = self.generate_uuid(False)[:16]
 
         original_attachment.filename = get_hashed_filename(
             original_attachment, image_type, key, uuid
@@ -294,7 +312,10 @@ def upload_image(self, image_paths: List[str], image_type: str) -> List[Attachme
             continue
 
         image_data = BytesIO()
-        image.save(image_data, format=x.file.format)
+        if x.file.format == "GIF" and x.file.is_animated:
+            x.file.save(image_data, format=x.file.format, save_all=True)
+        else:
+            x.file.save(image_data, format=x.file.format)
         image_data.seek(0)
 
         response = httpx.put(p_url, data=image_data.read())
@@ -323,7 +344,7 @@ def get_app_config(self) -> ApplicationConfig:
     return response
 
 
-def get_banned_words(self, country_code: str = "jp") -> List[BanWord]:
+def get_banned_words(self, country_code: str = "jp") -> list[BanWord]:
     response = self._make_request(
         "GET",
         endpoint=f"https://{Configs.YAY_CONFIG_HOST}/{country_code}/api/v2/banned_words",
@@ -332,7 +353,7 @@ def get_banned_words(self, country_code: str = "jp") -> List[BanWord]:
     return response
 
 
-def get_popular_words(self, country_code: str = "jp") -> List[PopularWord]:
+def get_popular_words(self, country_code: str = "jp") -> list[PopularWord]:
     response = self._make_request(
         "GET",
         endpoint=f"https://{Configs.YAY_CONFIG_HOST}/{country_code}/api/apps/yay/popular_words",
