@@ -1,7 +1,7 @@
 """
 MIT License
 
-Copyright (c) 2023-present Qvco, Konn
+Copyright (c) 2023-present qvco
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -22,9 +22,10 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 """
 
+from __future__ import annotations
+
 import json
 from datetime import datetime
-from typing import Union, Dict, List
 
 from ..config import Endpoints
 from ..errors import ForbiddenError, NotFoundError
@@ -44,11 +45,11 @@ from ..responses import (
 def add_bookmark(
     self, user_id: int, post_id: int, access_token: str = None
 ) -> BookmarkPostResponse:
-    self._check_authorization(access_token)
     response = self._make_request(
         "PUT",
         endpoint=f"{Endpoints.USERS_V1}/{user_id}/bookmarks/{post_id}",
         data_type=BookmarkPostResponse,
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Post has been added to the bookmarks.")
@@ -63,10 +64,10 @@ def add_group_highlight_post(
     投稿をグループのまとめに追加します
 
     """
-    self._check_authorization(access_token)
     response = self._make_request(
         "PUT",
         endpoint=f"{Endpoints.GROUPS_V1}/{group_id}/highlights/{post_id}",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Post has been added to the group highlight.")
@@ -95,8 +96,6 @@ def create_call_post(
     attachment_9_filename: str = None,
     access_token: str = None,
 ) -> ConferenceCall:
-    self._check_authorization(access_token)
-
     if text is not None:
         if "@:start:" in text and ":end:" in text:
             text, message_tags = parse_mention_format(self, text)
@@ -131,6 +130,7 @@ def create_call_post(
             "attachment_9_filename": attachment_9_filename,
         },
         data_type=CreatePostResponse,
+        auth_required=True,
         access_token=access_token,
     ).conference_call
     self.logger.info("Call post has been created.")
@@ -138,11 +138,11 @@ def create_call_post(
 
 
 def create_group_pin_post(self, post_id: int, group_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "PUT",
         endpoint=f"{Endpoints.POSTS_V2}/group_pinned_post",
         payload={"post_id": post_id, "group_id": group_id},
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Pinned the post in the group.")
@@ -150,11 +150,11 @@ def create_group_pin_post(self, post_id: int, group_id: int, access_token: str =
 
 
 def create_pin_post(self, post_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.PINNED_V1}/posts",
         payload={"id": post_id},
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Pinned post.")
@@ -223,8 +223,8 @@ def create_post(
     color: int = 0,
     in_reply_to: int = None,
     group_id: int = None,
-    mention_ids: List[int] = None,
-    choices: List[str] = None,
+    mention_ids: list[int] = None,
+    choices: list[str] = None,
     shared_url: str = None,
     message_tags: str = "[]",
     attachment_filename: str = None,
@@ -239,7 +239,6 @@ def create_post(
     video_file_name: str = None,
     access_token: str = None,
 ) -> Post:
-    self._check_authorization(access_token)
     headers = self.session.headers
     headers["X-Jwt"] = self.get_web_socket_token(access_token=access_token)
 
@@ -265,7 +264,7 @@ def create_post(
                 url=shared_url, access_token=access_token
             ).data
         except ForbiddenError:
-            self.logger.error("Unable to get the URL metadata")
+            self.logger.error("Unable to get the URL metadata.")
             shared_url = None
 
     response = self._make_request(
@@ -295,6 +294,7 @@ def create_post(
         },
         data_type=Post,
         headers=headers,
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Post has been created.")
@@ -309,9 +309,9 @@ def create_repost(
     color: int = None,
     in_reply_to: int = None,
     group_id: int = None,
-    mention_ids: List[int] = None,
-    choices: List[str] = None,
-    shared_url: Dict[str, Union[str, int]] = None,
+    mention_ids: list[int] = None,
+    choices: list[str] = None,
+    shared_url: dict[str, str | int] = None,
     message_tags: str = "[]",
     attachment_filename: str = None,
     attachment_2_filename: str = None,
@@ -325,7 +325,6 @@ def create_repost(
     video_file_name: str = None,
     access_token: str = None,
 ) -> Post:
-    self._check_authorization(access_token)
     headers = self.session.headers
     headers["X-Jwt"] = self.get_web_socket_token(access_token=access_token)
 
@@ -351,7 +350,7 @@ def create_repost(
                 url=shared_url, access_token=access_token
             ).data
         except ForbiddenError:
-            self.logger.error("Unable to get the URL metadata")
+            self.logger.error("Unable to get the URL metadata.")
             shared_url = None
 
     response = self._make_request(
@@ -382,6 +381,7 @@ def create_repost(
         },
         data_type=CreatePostResponse,
         headers=headers,
+        auth_required=True,
         access_token=access_token,
     ).post
     self.logger.info("Repost has been created.")
@@ -398,7 +398,6 @@ def create_share_post(
     group_id: int = None,
     access_token: str = None,
 ) -> Post:
-    self._check_authorization(access_token)
     timestamp = int(datetime.now().timestamp())
     response = self._make_request(
         "POST",
@@ -416,6 +415,7 @@ def create_share_post(
             "signed_info": self.generate_signed_info(self.device_uuid, timestamp),
         },
         data_type=Post,
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Share post has been created.")
@@ -430,9 +430,9 @@ def create_thread_post(
     color: int = None,
     in_reply_to: int = None,
     group_id: int = None,
-    mention_ids: List[int] = None,
-    choices: List[str] = None,
-    shared_url: Dict[str, Union[str, int]] = None,
+    mention_ids: list[int] = None,
+    choices: list[str] = None,
+    shared_url: dict[str, str | int] = None,
     message_tags: str = "[]",
     attachment_filename: str = None,
     attachment_2_filename: str = None,
@@ -446,7 +446,6 @@ def create_thread_post(
     video_file_name: str = None,
     access_token: str = None,
 ) -> Post:
-    self._check_authorization(access_token)
     headers = self.session.headers
     headers["X-Jwt"] = self.get_web_socket_token(access_token=access_token)
 
@@ -472,7 +471,7 @@ def create_thread_post(
                 url=shared_url, access_token=access_token
             ).data
         except ForbiddenError:
-            self.logger.error("Unable to get the URL metadata")
+            self.logger.error("Unable to get the URL metadata.")
             shared_url = None
 
     response = self._make_request(
@@ -503,6 +502,7 @@ def create_thread_post(
         },
         data_type=Post,
         headers=headers,
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Thread post has been created.")
@@ -510,11 +510,11 @@ def create_thread_post(
 
 
 def delete_all_post(self, access_token: str = None):
-    self._check_authorization(access_token)
     try:
         response = self._make_request(
             "POST",
             endpoint=f"{Endpoints.POSTS_V1}/delete_all_post",
+            auth_required=True,
             access_token=access_token,
         )
         self.logger.info("Post deletion request has been sent.")
@@ -524,11 +524,11 @@ def delete_all_post(self, access_token: str = None):
 
 
 def delete_group_pin_post(self, group_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "DELETE",
         endpoint=f"{Endpoints.POSTS_V2}/group_pinned_post",
         payload={"group_id": group_id},
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Unpinned post in the group.")
@@ -536,10 +536,10 @@ def delete_group_pin_post(self, group_id: int, access_token: str = None):
 
 
 def delete_pin_post(self, post_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "DELETE",
         endpoint=f"{Endpoints.PINNED_V1}/posts/{post_id}",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Unpinned post.")
@@ -549,7 +549,6 @@ def delete_pin_post(self, post_id: int, access_token: str = None):
 def get_bookmark(
     self, user_id: int, from_str: str = None, access_token: str = None
 ) -> PostsResponse:
-    self._check_authorization(access_token)
     params = {}
     if from_str:
         params = {"from": from_str}
@@ -558,6 +557,7 @@ def get_bookmark(
         endpoint=f"{Endpoints.USERS_V1}/{user_id}/bookmarks",
         params=params,
         data_type=PostsResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
@@ -614,7 +614,7 @@ def get_conversation(
 
 
 def get_conversation_root_posts(
-    self, post_ids: List[int], access_token: str = None
+    self, post_ids: list[int], access_token: str = None
 ) -> PostsResponse:
     return self._make_request(
         "GET",
@@ -641,12 +641,12 @@ def get_following_call_timeline(
         - exclude_recent_gomimushi: bool = None
 
     """
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.POSTS_V2}/call_followers_timeline",
         params=params,
         data_type=PostsResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
@@ -666,12 +666,12 @@ def get_following_timeline(self, access_token: str = None, **params) -> PostsRes
         - custom_generation_range: bool = None
 
     """
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.POSTS_V2}/following_timeline",
         params=params,
         data_type=PostsResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
@@ -784,12 +784,12 @@ def get_my_posts(self, access_token: str = None, **params) -> PostsResponse:
         - include_group_post: bool - (optional)
 
     """
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.POSTS_V2}/mine",
         params=params,
         data_type=PostsResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
@@ -846,7 +846,7 @@ def get_post_reposts(
     )
 
 
-def get_posts(self, post_ids: List[int], access_token: str = None) -> PostsResponse:
+def get_posts(self, post_ids: list[int], access_token: str = None) -> PostsResponse:
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.POSTS_V2}/multiple",
@@ -935,13 +935,13 @@ def get_timeline(
     """
     endpoint = f"{Endpoints.POSTS_V2}/timeline"
     if "noreply_mode" in params and params["noreply_mode"] is True:
-        self._check_authorization(access_token)
         endpoint = f"{Endpoints.POSTS_V2}/noreply_timeline"
     return self._make_request(
         "GET",
         endpoint=endpoint,
         params=params,
         data_type=PostsResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
@@ -980,14 +980,14 @@ def get_user_timeline(
 
 
 def like_posts(
-    self, post_ids: List[int], access_token: str = None
+    self, post_ids: list[int], access_token: str = None
 ) -> LikePostsResponse:
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.POSTS_V2}/like",
         payload={"post_ids": post_ids},
         data_type=LikePostsResponse,
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Posts have been liked.")
@@ -995,10 +995,10 @@ def like_posts(
 
 
 def remove_bookmark(self, user_id: int, post_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "DELETE",
         endpoint=f"{Endpoints.USERS_V1}/{user_id}/bookmarks/{post_id}",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Bookmark has been removed.")
@@ -1008,22 +1008,22 @@ def remove_bookmark(self, user_id: int, post_id: int, access_token: str = None):
 def remove_group_highlight_post(
     self, group_id: int, post_id: int, access_token: str = None
 ):
-    self._check_authorization(access_token)
     response = self._make_request(
         "DELETE",
         endpoint=f"{Endpoints.GROUPS_V1}/{group_id}/highlights/{post_id}",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Group hightlight post removed.")
     return response
 
 
-def remove_posts(self, post_ids: List[int], access_token: str = None):
-    self._check_authorization(access_token)
+def remove_posts(self, post_ids: list[int], access_token: str = None):
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.POSTS_V2}/mass_destroy",
         payload={"posts_ids": post_ids},
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Posts have been removed.")
@@ -1042,7 +1042,6 @@ def report_post(
     screenshot_4_filename: str = None,
     access_token: str = None,
 ):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.POSTS_V3}/{post_id}/report",
@@ -1055,6 +1054,7 @@ def report_post(
             "screenshot_3_filename": screenshot_3_filename,
             "screenshot_4_filename": screenshot_4_filename,
         },
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Post has been reported.")
@@ -1062,10 +1062,10 @@ def report_post(
 
 
 def unlike_post(self, post_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.POSTS_V1}/{post_id}/unlike",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Post has been unliked.")
@@ -1081,8 +1081,6 @@ def update_post(
     message_tags: str = "[]",
     access_token: str = None,
 ) -> Post:
-    self._check_authorization(access_token)
-
     if "@:start:" in text and ":end:" in text:
         text, message_tags = parse_mention_format(self, text)
 
@@ -1100,6 +1098,7 @@ def update_post(
             "timestamp": timestamp,
             "signed_info": self.generate_signed_info(self.device_uuid, timestamp),
         },
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Post has been updated.")
@@ -1119,12 +1118,12 @@ def view_video(self, video_id: int, access_token: str = None):
 def vote_survey(
     self, survey_id: int, choice_id: int, access_token: str = None
 ) -> Survey:
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.SURVEYS_V2}/{survey_id}/vote",
         payload={"choice_id": choice_id},
         data_type=ValidationPostResponse,
+        auth_required=True,
         access_token=access_token,
     ).survey
     self.logger.info("Voted.")
