@@ -1,7 +1,7 @@
 """
 MIT License
 
-Copyright (c) 2023-present Qvco, Konn
+Copyright (c) 2023-present qvco
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -22,27 +22,27 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 """
 
+from __future__ import annotations
+
 from datetime import datetime
-from typing import List
 
 from ..config import Endpoints
 from ..responses import ReviewsResponse
 
 
 def create_review(self, user_id: int, comment: str, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.USERS_V1}/reviews/{user_id}",
         payload={"comment": comment},
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info(f"Review has been sent to {user_id}.")
     return response
 
 
-def create_reviews(self, user_ids: List[int], comment: str, access_token: str = None):
-    self._check_authorization(access_token)
+def create_reviews(self, user_ids: list[int], comment: str, access_token: str = None):
     timestamp = int(datetime.now().timestamp())
     response = self._make_request(
         "POST",
@@ -57,18 +57,19 @@ def create_reviews(self, user_ids: List[int], comment: str, access_token: str =
                 self.uuid, timestamp, shared_key=True
             ),
         },
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Reviews have been sent to multiple users.")
     return response
 
 
-def delete_reviews(self, review_ids: List[int], access_token: str = None):
-    self._check_authorization(access_token)
+def delete_reviews(self, review_ids: list[int], access_token: str = None):
     response = self._make_request(
         "DELETE",
         endpoint=f"{Endpoints.USERS_V1}/reviews",
         params={"review_ids[]": review_ids},
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Reviews have been deleted.")
@@ -85,12 +86,12 @@ def get_my_reviews(self, access_token: str = None, **params) -> ReviewsResponse:
         - number: int = (optional)
 
     """
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.USERS_V1}/reviews/mine",
         params=params,
         data_type=ReviewsResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
@@ -118,11 +119,11 @@ def get_reviews(
 
 
 def pin_review(self, review_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.PINNED_V1}/reviews",
         payload={"id": review_id},
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Pinned the review.")
@@ -130,10 +131,10 @@ def pin_review(self, review_id: int, access_token: str = None):
 
 
 def unpin_review(self, review_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "DELETE",
         endpoint=f"{Endpoints.PINNED_V1}/reviews{review_id}",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Unpinned the review.")
