@@ -1,7 +1,7 @@
 """
 MIT License
 
-Copyright (c) 2023-present Qvco, Konn
+Copyright (c) 2023-present qvco
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -22,7 +22,7 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 """
 
-from typing import List
+from __future__ import annotations
 
 from ..config import Endpoints
 from ..models import ThreadInfo
@@ -32,11 +32,11 @@ from ..responses import GroupThreadListResponse, PostsResponse
 def add_post_to_thread(
     self, post_id: int, thread_id: int, access_token: str = None
 ) -> ThreadInfo:
-    self._check_authorization(access_token)
     response = self._make_request(
         "PUT",
         endpoint=f"{Endpoints.POSTS_V3}/{post_id}/move_to_thread/{thread_id}",
         data_type=ThreadInfo,
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info(f"Post '{post_id}' added to the thread '{thread_id}'.")
@@ -50,12 +50,12 @@ def convert_post_to_thread(
     thread_icon_filename: str = None,
     access_token: str = None,
 ) -> ThreadInfo:
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.POSTS_V3}/{post_id}/move_to_thread",
         payload={"title": title, "thread_icon_filename": thread_icon_filename},
         data_type=ThreadInfo,
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Post has been converted to a thread.")
@@ -65,7 +65,6 @@ def convert_post_to_thread(
 def create_thread(
     self, group_id: int, title: str, thread_icon_filename: str, access_token: str = None
 ) -> ThreadInfo:
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.THREADS_V1}",
@@ -75,6 +74,7 @@ def create_thread(
             "thread_icon_filename": thread_icon_filename,
         },
         data_type=ThreadInfo,
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("A new thread has been created.")
@@ -106,12 +106,12 @@ def get_group_thread_list(
     )
 
 
-def get_thread_joined_statuses(self, ids: List[int], access_token: str = None) -> dict:
-    self._check_authorization(access_token)
+def get_thread_joined_statuses(self, ids: list[int], access_token: str = None) -> dict:
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.THREADS_V1}/joined_statuses",
         params={"ids[]": ids},
+        auth_required=True,
         access_token=access_token,
     )
 
@@ -141,10 +141,10 @@ def get_thread_posts(
 
 
 def join_thread(self, thread_id: int, user_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.THREADS_V1}/{thread_id}/members/{user_id}",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info(f"Joined the thread '{thread_id}'.")
@@ -152,10 +152,10 @@ def join_thread(self, thread_id: int, user_id: int, access_token: str = None):
 
 
 def leave_thread(self, thread_id: int, user_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "DELETE",
         endpoint=f"{Endpoints.THREADS_V1}/{thread_id}/members/{user_id}",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Left the thread.")
@@ -163,10 +163,10 @@ def leave_thread(self, thread_id: int, user_id: int, access_token: str = None):
 
 
 def remove_thread(self, thread_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "DELETE",
         endpoint=f"{Endpoints.THREADS_V1}/{thread_id}",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info(f"Thread '{thread_id}' has been removed.")
@@ -180,11 +180,11 @@ def update_thread(
     thread_icon_filename: str,
     access_token: str = None,
 ):
-    self._check_authorization(access_token)
     response = self._make_request(
         "PUT",
         endpoint=f"{Endpoints.THREADS_V1}/{thread_id}",
         payload={"title": title, "thread_icon_filename": thread_icon_filename},
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info(f"Thread '{thread_id}' has been updated.")
