@@ -1,7 +1,7 @@
 """
 MIT License
 
-Copyright (c) 2023-present Qvco, Konn
+Copyright (c) 2023-present qvco
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -22,8 +22,9 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 """
 
+from __future__ import annotations
+
 from datetime import datetime
-from typing import Union, List
 
 from ..config import Configs, Endpoints
 from ..models import Footprint, User, UserWrapper
@@ -51,10 +52,10 @@ from ..responses import (
 
 
 def delete_footprint(self, user_id: int, footprint_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "DELETE",
         endpoint=f"{Endpoints.USERS_V2}/{user_id}/footprints/{footprint_id}",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Footprint has been deleted.")
@@ -62,10 +63,6 @@ def delete_footprint(self, user_id: int, footprint_id: int, access_token: str =
 
 
 def destroy_user(self, access_token: str = None):
-    self._check_authorization(access_token)
-    answer = input("Are you sure you want to delete your account? Y/N")
-    if answer.lower() != "y":
-        return
     timestamp = int(datetime.now().timestamp())
     response = self._make_request(
         "POST",
@@ -76,6 +73,7 @@ def destroy_user(self, access_token: str = None):
             "timestamp": timestamp,
             "signed_info": self.generate_signed_info(self.device_uuid, timestamp),
         },
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("User has been deleted.")
@@ -83,22 +81,22 @@ def destroy_user(self, access_token: str = None):
 
 
 def follow_user(self, user_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.USERS_V2}/{user_id}/follow",
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info(f"Followed the user '{user_id}'.")
     return response
 
 
-def follow_users(self, user_ids: List[int], access_token: str = None):
-    self._check_authorization(access_token)
+def follow_users(self, user_ids: list[int], access_token: str = None):
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.USERS_V2}/follow",
         params={"user_ids[]": user_ids},
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("Followed multiple users.")
@@ -117,12 +115,12 @@ def get_active_followings(
         - from_loggedin_at: int = None
 
     """
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.USERS_V1}/active_followings",
         params=params,
         data_type=ActiveFollowingsResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
@@ -137,15 +135,15 @@ def get_follow_recommendations(
 
         - from_timestamp: int = None,
         - number: int = None,
-        - sources: List[str] = None
+        - sources: list[str] = None
 
     """
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.FRIENDS_V1}",
         params=params,
         data_type=FollowRecommendationsResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
@@ -156,22 +154,22 @@ def get_follow_request(
     params = {}
     if from_timestamp:
         params["from_timestamp"] = from_timestamp
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.USERS_V2}/follow_requests",
         params=params,
         data_type=UsersByTimestampResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
 
 def get_follow_request_count(self, access_token: str = None) -> int:
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.USERS_V2}/follow_requests_count",
         data_type=FollowRequestCountResponse,
+        auth_required=True,
         access_token=access_token,
     ).users_count
 
@@ -182,17 +180,17 @@ def get_following_users_born(
     params = {}
     if birthdate:
         params["birthdate"] = birthdate
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.USERS_V1}/following_born_today",
         params=params,
         data_type=UsersResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
 
-def get_footprints(self, access_token: str = None, **params) -> List[Footprint]:
+def get_footprints(self, access_token: str = None, **params) -> list[Footprint]:
     """
 
     Parameters:
@@ -203,27 +201,27 @@ def get_footprints(self, access_token: str = None, **params) -> List[Footprint]:
         - mode: str = None
 
     """
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.USERS_V2}/footprints",
         params=params,
         data_type=FootprintsResponse,
+        auth_required=True,
         access_token=access_token,
     ).footprints
 
 
 def get_fresh_user(self, user_id: int, access_token: str = None) -> UserResponse:
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.USERS_V2}/fresh/{user_id}",
         data_type=UserResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
 
-def get_hima_users(self, access_token: str = None, **params) -> List[UserWrapper]:
+def get_hima_users(self, access_token: str = None, **params) -> list[UserWrapper]:
     """
 
     Parameters:
@@ -233,12 +231,12 @@ def get_hima_users(self, access_token: str = None, **params) -> List[UserWrapper
         - number: int = None
 
     """
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.USERS_V2}/hima_users",
         params=params,
         data_type=HimaUsersResponse,
+        auth_required=True,
         access_token=access_token,
     ).hima_users
 
@@ -279,11 +277,11 @@ def get_user_ranking(self, mode: str) -> RankingUsersResponse:
 def get_refresh_counter_requests(
     self, access_token: str = None
 ) -> RefreshCounterRequestsResponse:
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.USERS_V1}/reset_counters",
         data_type=RefreshCounterRequestsResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
@@ -301,12 +299,12 @@ def get_social_shared_users(
         - from_id: int - (Optional)
 
     """
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.USERS_V2}/social_shared_users",
         params=params,
         data_type=SocialShareUsersResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
@@ -316,6 +314,7 @@ def get_timestamp(self, access_token: str = None) -> UserTimestampResponse:
         "GET",
         endpoint=f"{Endpoints.USERS_V2}/timestamp",
         data_type=UserTimestampResponse,
+        bypass_delay=True,
         access_token=access_token,
     )
 
@@ -330,11 +329,11 @@ def get_user(self, user_id: int, access_token: str = None) -> User:
 
 
 def get_user_email(self, user_id: int, access_token: str = None) -> str:
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.USERS_V2}/fresh/{user_id}",
         data_type=UserEmailResponse,
+        auth_required=True,
         access_token=access_token,
     ).email
 
@@ -378,12 +377,12 @@ def get_user_followings(
         - number: int - (optional)
 
     """
-    self._check_authorization(access_token)
     return self._make_request(
         "POST",
         endpoint=f"{Endpoints.USERS_V2}/{user_id}/list_followings",
         params=params,
         data_type=FollowUsersResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
@@ -408,7 +407,7 @@ def get_user_without_leaving_footprint(
     )
 
 
-def get_users(self, user_ids: List[int], access_token: str = None) -> UsersResponse:
+def get_users(self, user_ids: list[int], access_token: str = None) -> UsersResponse:
     headers = self.session.headers
     headers["X-Jwt"] = self.get_web_socket_token(access_token=access_token)
     return self._make_request(
@@ -511,7 +510,6 @@ def report_user(
     screenshot_4_filename: str = None,
     access_token: str = None,
 ):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.USERS_V3}/{user_id}/report",
@@ -523,9 +521,10 @@ def report_user(
             "screenshot_3_filename": screenshot_3_filename,
             "screenshot_4_filename": screenshot_4_filename,
         },
+        auth_required=True,
         access_token=access_token,
     )
-    self.logger.info(f"Reported the user '{user_id}'")
+    self.logger.info(f"Reported the user '{user_id}'.")
     return response
 
 
@@ -640,22 +639,24 @@ def take_action_follow_request(
 
 
 def turn_on_hima(self, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
-        "GET", endpoint=f"{Endpoints.USERS_V1}/hima", access_token=access_token
+        "GET",
+        endpoint=f"{Endpoints.USERS_V1}/hima",
+        auth_required=True,
+        access_token=access_token,
     )
     self.logger.info("Turned on 'hima now'.")
     return response
 
 
 def unfollow_user(self, user_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.USERS_V2}/{user_id}/unfollow",
+        auth_required=True,
         access_token=access_token,
     )
-    self.logger.info(f"Unfollowed the user '{user_id}'")
+    self.logger.info(f"Unfollowed the user '{user_id}'.")
     return response
 
 
@@ -677,37 +678,42 @@ def update_language(self, language: str, access_token: str = None):
     return response
 
 
-def update_user(
-    self,
-    nickname: str,
-    biography: str = None,
-    prefecture: str = None,
-    gender: int = None,
-    country_code: str = None,
-    profile_icon_filename: str = None,
-    cover_image_filename: str = None,
-    username: str = None,
-    access_token: str = None,
-):
-    self._check_authorization(access_token)
+def update_user(self, nickname: str, access_token: str = None, **params):
+    """
+
+    プロフィールを更新します
+
+    Parameters
+    ----------
+
+        - nickname: str = (required)
+        - biography: str = (optional)
+        - prefecture: str = (optional)
+        - gender: int = (optional)
+        - country_code: str = (optional)
+        - profile_icon_filename: str = (optional)
+        - cover_image_filename: str = (optional)
+        - username: str = (optional)
+        - access_token: str = (optional)
+
+    """
     timestamp = int(datetime.now().timestamp())
-    response = self._make_request(
-        "POST",
-        endpoint=f"{Endpoints.USERS_V3}/edit",
-        payload={
+    params.update(
+        {
             "nickname": nickname,
-            "biography": biography,
-            "prefecture": prefecture,
-            "gender": gender,
-            "country_code": country_code,
-            "profile_icon_filename": profile_icon_filename,
-            "cover_image_filename": cover_image_filename,
-            "username": username,
             "uuid": self.uuid,
             "api_key": self.api_key,
             "timestamp": timestamp,
-            "signed_info": self.generate_signed_info(self.device_uuid, timestamp),
-        },
+            "signed_info": self.generate_signed_info(
+                self.uuid, timestamp, shared_key=True
+            ),
+        }
+    )
+    response = self._make_request(
+        "POST",
+        endpoint=f"{Endpoints.USERS_V3}/edit",
+        payload=params,
+        auth_required=True,
         access_token=access_token,
     )
     self.logger.info("User profile has been updated.")
@@ -718,22 +724,22 @@ def update_user(
 
 
 def block_user(self, user_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.USERS_V1}/{user_id}/block",
+        auth_required=True,
         access_token=access_token,
     )
-    self.logger.info(f"Blocked the user '{user_id}'")
+    self.logger.info(f"Blocked the user '{user_id}'.")
     return response
 
 
 def get_blocked_user_ids(self, access_token: str = None) -> BlockedUserIdsResponse:
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.USERS_V1}/block_ids",
         data_type=BlockedUserIdsResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
@@ -742,7 +748,6 @@ def get_blocked_users(
     self, from_id: int = None, access_token: str = None
 ) -> BlockedUsersResponse:
     # @Body @NotNull SearchUsersRequest searchUsersRequest
-    self._check_authorization(access_token)
     params = {}
     if from_id:
         params["from_id"] = from_id
@@ -751,27 +756,26 @@ def get_blocked_users(
         endpoint=f"{Endpoints.USERS_V2}/blocked",
         params=params,
         data_type=BlockedUsersResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
 
 def unblock_user(self, user_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.USERS_V2}/{user_id}/unblock",
+        auth_required=True,
         access_token=access_token,
     )
-    self.logger.info(f"Unblocked the user '{user_id}'")
+    self.logger.info(f"Unblocked the user '{user_id}'.")
     return response
 
 
 # HiddenApi
 
 
-def get_hidden_users_list(
-    self, access_token: str = None, **params: Union[str, int]
-) -> HiddenResponse:
+def get_hidden_users_list(self, access_token: str = None, **params) -> HiddenResponse:
     """
 
     Parameters:
@@ -781,35 +785,35 @@ def get_hidden_users_list(
         - number: int = None
 
     """
-    self._check_authorization(access_token)
     return self._make_request(
         "GET",
         endpoint=f"{Endpoints.HIDDEN_V1}/users",
         params=params,
         data_type=HiddenResponse,
+        auth_required=True,
         access_token=access_token,
     )
 
 
 def hide_user(self, user_id: int, access_token: str = None):
-    self._check_authorization(access_token)
     response = self._make_request(
         "POST",
         endpoint=f"{Endpoints.HIDDEN_V1}/users",
         payload={"user_id": user_id},
+        auth_required=True,
         access_token=access_token,
     )
-    self.logger.info(f"User '{user_id}' is hidden")
+    self.logger.info(f"User '{user_id}' is hidden.")
     return response
 
 
-def unhide_users(self, user_ids: List[int], access_token: str = None):
-    self._check_authorization(access_token)
+def unhide_users(self, user_ids: list[int], access_token: str = None):
     response = self._make_request(
         "DELETE",
         endpoint=f"{Endpoints.HIDDEN_V1}/users",
         params={"user_ids[]": user_ids},
+        auth_required=True,
         access_token=access_token,
     )
-    self.logger.info("Unhid users")
+    self.logger.info("Unhid users.")
     return response
