@@ -1,7 +1,7 @@
 """
 MIT License
 
-Copyright (c) 2023-present Qvco, Konn
+Copyright (c) 2023-present qvco
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -22,7 +22,7 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 """
 
-from typing import Dict, List, Union
+from __future__ import annotations
 
 from .api import API
 from .api.call import (
@@ -127,11 +127,11 @@ from .api.login import (
     change_email,
     change_password,
     get_token,
-    is_valid_token,
-    login_with_email,
+    login_flow,
     logout,
     resend_confirm_email,
     restore_user,
+    register_device_token,
     revoke_tokens,
     save_account_with_email,
 )
@@ -144,6 +144,7 @@ from .api.misc import (
     get_file_upload_presigned_urls,
     get_id_checker_presigned_url,
     get_old_file_upload_presigned_url,
+    get_policy_agreements,
     get_web_socket_token,
     verify_device,
     upload_image,
@@ -311,11 +312,13 @@ from .responses import (
     LoginUserResponse,
     LoginUpdateResponse,
     MessageResponse,
+    PolicyAgreementsResponse,
     PostsResponse,
     PostLikersResponse,
     PostTagsResponse,
     LikePostsResponse,
     RefreshCounterRequestsResponse,
+    RegisterDeviceTokenResponse,
     ReviewsResponse,
     SocialShareUsersResponse,
     TokenResponse,
@@ -326,6 +329,7 @@ from .responses import (
     UsersByTimestampResponse,
     UserTimestampResponse,
 )
+from .utils import Colors, console_print
 
 
 POST_TYPE_TEXT = "text"
@@ -408,6 +412,9 @@ class Client(API):
         - proxy: str - (optional)
         - max_retries: int - (optional)
         - backoff_factor: float - (optional)
+        - wait_on_rate_limit: bool - (optional)
+        - min_delay: float - (optional)
+        - max_delay: float - (optional)
         - timeout: int - (optional)
         - err_lang: str - (optional)
         - base_path: str - (optional)
@@ -422,7 +429,7 @@ class Client(API):
 
     ### Yay! (nanameue, Inc.) API Client
 
-    Copyright (c) 2023-present Qvco, Konn
+    Copyright (c) 2023-present qvco
 
     """
 
@@ -436,7 +443,7 @@ class Client(API):
         """
         return get_user_active_call(self, user_id, access_token)
 
-    def get_bgms(self, access_token: str = None) -> List[Bgm]:
+    def get_bgms(self, access_token: str = None) -> list[Bgm]:
         """
 
         通話のBGMを取得します
@@ -455,7 +462,6 @@ class Client(API):
     def get_call_invitable_users(
         self, call_id: int, from_timestamp: int = None, access_token: str = None
     ) -> UsersByTimestampResponse:
-        # @Nullable @Query("user[nickname]")
         """
 
         通話に招待可能なユーザーを取得します
@@ -481,7 +487,7 @@ class Client(API):
         Parameters
         ----------
             - number: int - (optional)
-            - ids: List[int] - (optional)
+            - ids: list[int] - (optional)
             - from_id: int - (optional)
 
         """
@@ -531,7 +537,7 @@ class Client(API):
         return invite_to_call_bulk(self, call_id, group_id, access_token)
 
     def invite_users_to_call(
-        self, call_id: int, user_ids: List[int], access_token: str = None
+        self, call_id: int, user_ids: list[int], access_token: str = None
     ) -> dict:
         """
 
@@ -540,7 +546,7 @@ class Client(API):
         Parameters
         ----------
             - call_id: int - (required)
-            - user_ids: List[int] - (required)
+            - user_ids: list[int] - (required)
 
         """
         return invite_users_to_call(self, call_id, user_ids, access_token)
@@ -685,7 +691,7 @@ class Client(API):
     # -CHAT
 
     def accept_chat_requests(
-        self, chat_room_ids: List[int], access_token: str = None
+        self, chat_room_ids: list[int], access_token: str = None
     ) -> dict:
         """
 
@@ -694,7 +700,7 @@ class Client(API):
         Parameters
         ----------
 
-            - chat_room_ids: List[int] - (required)
+            - chat_room_ids: list[int] - (required)
 
         """
         return accept_chat_requests(self, chat_room_ids, access_token)
@@ -712,7 +718,7 @@ class Client(API):
     def create_group_chat(
         self,
         name: str,
-        with_user_ids: List[int],
+        with_user_ids: list[int],
         icon_filename: str = None,
         background_filename: str = None,
         access_token: str = None,
@@ -725,7 +731,7 @@ class Client(API):
         ----------
 
             - name: str - (required)
-            - with_user_ids: List[int] - (required)
+            - with_user_ids: list[int] - (required)
             - icon_filename: str - (optional)
             - background_filename: str - (optional)
 
@@ -819,7 +825,7 @@ class Client(API):
             from_follow_id, from_timestamp, order_by, access_token
         )
 
-    def get_gifs_data(self, access_token: str = None) -> List[GifImageCategory]:
+    def get_gifs_data(self, access_token: str = None) -> list[GifImageCategory]:
         """
 
         チャットルームのGIFデータを取得します
@@ -855,7 +861,7 @@ class Client(API):
 
     def get_messages(
         self, chat_room_id: int, access_token: str = None, **params
-    ) -> List[Message]:
+    ) -> list[Message]:
         """
 
         メッセージを取得します
@@ -893,7 +899,7 @@ class Client(API):
         """
         return get_chat_room(self, chat_room_id, access_token)
 
-    def get_sticker_packs(self, access_token: str = None) -> List[StickerPack]:
+    def get_sticker_packs(self, access_token: str = None) -> list[StickerPack]:
         """
 
         スタンプを取得します
@@ -918,7 +924,7 @@ class Client(API):
         return hide_chat(self, chat_room_id, access_token)
 
     def invite_to_chat(
-        self, chat_room_id: int, user_ids: List[int], access_token: str = None
+        self, chat_room_id: int, user_ids: list[int], access_token: str = None
     ) -> dict:
         """
 
@@ -928,7 +934,7 @@ class Client(API):
         return invite_to_chat(self, chat_room_id, user_ids, access_token)
 
     def kick_users_from_chat(
-        self, chat_room_id: int, user_ids: List[int], access_token: str = None
+        self, chat_room_id: int, user_ids: list[int], access_token: str = None
     ) -> dict:
         """
 
@@ -966,7 +972,7 @@ class Client(API):
         return refresh_chat_rooms(self, from_time, access_token)
 
     def remove_chat_rooms(
-        self, chat_room_ids: List[int], access_token: str = None
+        self, chat_room_ids: list[int], access_token: str = None
     ) -> dict:
         """
 
@@ -1008,36 +1014,31 @@ class Client(API):
     def send_message(
         self,
         chat_room_id: int,
-        message_type: str,
-        call_type: str = None,
-        text: str = None,
-        font_size: int = None,
-        gif_image_id: int = None,
-        attachment_file_name: str = None,
-        sticker_pack_id: int = None,
-        sticker_id: int = None,
-        video_file_name: str = None,
         access_token: str = None,
+        **params,
     ) -> MessageResponse:
         """
 
         チャットルームにメッセージを送信します
 
+        Parameters:
+        ----------
+
+            - chat_room_id: int (required)
+            - message_type: str (required)
+            - call_type: str = (optional)
+            - text: str = (optional)
+            - font_size: int = (optional)
+            - gif_image_id: int = (optional)
+            - attachment_file_name: str = (optional)
+            - sticker_pack_id: int = (optional)
+            - sticker_id: int = (optional)
+            - video_file_name: str = (optional)
+            - parent_id: int = (optional)
+            - access_token: str = (optional)
+
         """
-        return send_message(
-            self,
-            chat_room_id,
-            message_type,
-            call_type,
-            text,
-            font_size,
-            gif_image_id,
-            attachment_file_name,
-            sticker_pack_id,
-            sticker_id,
-            video_file_name,
-            access_token,
-        )
+        return send_message(self, chat_room_id, access_token, **params)
 
     def unhide_chat(self, chat_room_ids: int, access_token: str = None) -> dict:
         """
@@ -1084,7 +1085,7 @@ class Client(API):
         return accept_group_join_request(self, group_id, user_id, access_token)
 
     def add_related_groups(
-        self, group_id: int, related_group_id: List[int], access_token: str = None
+        self, group_id: int, related_group_id: list[int], access_token: str = None
     ) -> dict:
         """
 
@@ -1283,7 +1284,7 @@ class Client(API):
         """
         return get_invitable_users(self, group_id, access_token, **params)
 
-    def get_joined_statuses(self, ids: List[int], access_token: str = None) -> dict:
+    def get_joined_statuses(self, ids: list[int], access_token: str = None) -> dict:
         """
 
         サークルの参加ステータスを取得します
@@ -1381,7 +1382,7 @@ class Client(API):
         return get_user_groups(self, access_token, **params)
 
     def invite_users_to_group(
-        self, group_id: int, user_ids: List[int], access_token: str = None
+        self, group_id: int, user_ids: list[int], access_token: str = None
     ) -> dict:
         """
 
@@ -1435,7 +1436,7 @@ class Client(API):
         return remove_moderator(self, group_id, user_id, access_token)
 
     def remove_related_groups(
-        self, group_id: int, related_group_ids: List[int], access_token: str = None
+        self, group_id: int, related_group_ids: list[int], access_token: str = None
     ) -> dict:
         """
 
@@ -1475,7 +1476,7 @@ class Client(API):
         )
 
     def send_moderator_offers(
-        self, group_id: int, user_ids: List[int], access_token: str = None
+        self, group_id: int, user_ids: list[int], access_token: str = None
     ) -> dict:
         """
 
@@ -1601,14 +1602,6 @@ class Client(API):
 
     # -LOGIN
 
-    def is_valid_token(self, access_token: str):
-        """
-
-        アクセストークンが有効か検証します
-
-        """
-        return is_valid_token(self, access_token)
-
     def change_password(
         self, current_password: str, new_password: str, access_token: str = None
     ) -> LoginUpdateResponse:
@@ -1643,9 +1636,16 @@ class Client(API):
 
         ※ ローカルストレージのトークンの暗号化を利用するには、`Client` クラスの `encrypt_cookie` 引数を`True` に設定してください。
 
-
         """
-        return login_with_email(self, email, password, secret_key)
+        login_response = login_flow(self, email, password, secret_key)
+
+        policy_response = get_policy_agreements(self)
+        if not policy_response.latest_privacy_policy_agreed:
+            accept_policy_agreement(self, type="privacy_policy")
+        if not policy_response.latest_terms_of_use_agreed:
+            accept_policy_agreement(self, type="terms_of_use")
+
+        return login_response
 
     def logout(self, access_token: str = None) -> dict:
         """
@@ -1671,6 +1671,38 @@ class Client(API):
         """
         return restore_user(self, user_id, access_token)
 
+    def register_device_token(
+        self,
+        device_token: str,
+        device_type: str,
+        os_version: str,
+        app_version: str,
+        screen_resolution: str,
+        screen_density: str,
+        device_model: str,
+        appsflyer_id: str,
+        advertising_id: str = None,
+        access_token: str = None,
+    ) -> RegisterDeviceTokenResponse:
+        """
+
+        デバイストークンを登録します
+
+        """
+        return register_device_token(
+            self,
+            device_token,
+            device_type,
+            os_version,
+            app_version,
+            screen_resolution,
+            screen_density,
+            device_model,
+            appsflyer_id,
+            advertising_id,
+            access_token,
+        )
+
     def revoke_tokens(self, access_token: str = None) -> dict:
         """
 
@@ -1703,6 +1735,14 @@ class Client(API):
 
     # -MISC
 
+    def accept_policy_agreement(self, type: str, access_token: str = None):
+        """
+
+        利用規約、ポリシー同意書に同意します
+
+        """
+        return accept_policy_agreement(self, type, access_token)
+
     def send_verification_code(self, email: str):
         """
 
@@ -1732,8 +1772,8 @@ class Client(API):
         )
 
     def get_file_upload_presigned_urls(
-        self, file_names: List[str], access_token: str = None
-    ) -> List[PresignedUrl]:
+        self, file_names: list[str], access_token: str = None
+    ) -> list[PresignedUrl]:
         """
 
         ファイルアップロード用の署名付きURLを取得します
@@ -1759,6 +1799,16 @@ class Client(API):
         """
         return get_old_file_upload_presigned_url(self, video_file_name, access_token)
 
+    def get_policy_agreements(
+        self, access_token: str = None
+    ) -> PolicyAgreementsResponse:
+        """
+
+        利用規約、ポリシー同意書に同意しているかどうかを取得します
+
+        """
+        return get_policy_agreements(self, access_token)
+
     def get_web_socket_token(
         self, headers: dict = None, access_token: str = None
     ) -> str:
@@ -1769,7 +1819,7 @@ class Client(API):
         """
         return get_web_socket_token(self, headers, access_token)
 
-    def upload_image(self, image_paths: List[str], image_type: str) -> List[Attachment]:
+    def upload_image(self, image_paths: list[str], image_type: str) -> list[Attachment]:
         """
 
         画像をアップロードして、サーバー上のファイル名のリストを返します。
@@ -1777,7 +1827,7 @@ class Client(API):
         Parameteres
         -----------
 
-            - image_path: List[str] - (required): 画像パスのリスト
+            - image_path: list[str] - (required): 画像パスのリスト
             - image_type: str - (required): 画像の種類
 
         Examples
@@ -1807,7 +1857,7 @@ class Client(API):
         """
         return get_app_config(self)
 
-    def get_banned_words(self, country_code: str = "jp") -> List[BanWord]:
+    def get_banned_words(self, country_code: str = "jp") -> list[BanWord]:
         """
 
         禁止ワードの一覧を取得します
@@ -1815,7 +1865,7 @@ class Client(API):
         """
         return get_banned_words(self, country_code)
 
-    def get_popular_words(self, country_code: str = "jp") -> List[PopularWord]:
+    def get_popular_words(self, country_code: str = "jp") -> list[PopularWord]:
         """
 
         人気のワードの一覧を取得します
@@ -1928,8 +1978,8 @@ class Client(API):
         color: int = 0,
         in_reply_to: int = None,
         group_id: int = None,
-        mention_ids: List[int] = None,
-        choices: List[str] = None,
+        mention_ids: list[int] = None,
+        choices: list[str] = None,
         shared_url: str = None,
         message_tags: str = "[]",
         attachment_filename: str = None,
@@ -1981,9 +2031,9 @@ class Client(API):
         color: int = None,
         in_reply_to: int = None,
         group_id: int = None,
-        mention_ids: List[int] = None,
-        choices: List[str] = None,
-        shared_url: Dict[str, Union[str, int]] = None,
+        mention_ids: list[int] = None,
+        choices: list[str] = None,
+        shared_url: dict[str, str | int] = None,
         message_tags: str = "[]",
         attachment_filename: str = None,
         attachment_2_filename: str = None,
@@ -2061,9 +2111,9 @@ class Client(API):
         color: int = None,
         in_reply_to: int = None,
         group_id: int = None,
-        mention_ids: List[int] = None,
-        choices: List[str] = None,
-        shared_url: Dict[str, Union[str, int]] = None,
+        mention_ids: list[int] = None,
+        choices: list[str] = None,
+        shared_url: dict[str, str | int] = None,
         message_tags: str = "[]",
         attachment_filename: str = None,
         attachment_2_filename: str = None,
@@ -2183,7 +2233,7 @@ class Client(API):
         return get_conversation(self, conversation_id, access_token, **params)
 
     def get_conversation_root_posts(
-        self, post_ids: List[int], access_token: str = None
+        self, post_ids: list[int], access_token: str = None
     ) -> PostsResponse:
         """
 
@@ -2364,7 +2414,7 @@ class Client(API):
         """
         return get_post_reposts(self, post_id, access_token, **params)
 
-    def get_posts(self, post_ids: List[int], access_token: str = None) -> PostsResponse:
+    def get_posts(self, post_ids: list[int], access_token: str = None) -> PostsResponse:
         """
 
         複数の投稿を取得します
@@ -2420,7 +2470,6 @@ class Client(API):
         return get_timeline_by_keyword(self, keyword, access_token, **params)
 
     def get_timeline(self, access_token: str = None, **params) -> PostsResponse:
-        # - from: str - (optional)
         """
 
         タイムラインを取得します
@@ -2468,7 +2517,7 @@ class Client(API):
         """
         return get_user_timeline(self, user_id, access_token, **params)
 
-    def like(self, post_ids: List[int], access_token: str = None) -> LikePostsResponse:
+    def like(self, post_ids: list[int], access_token: str = None) -> LikePostsResponse:
         """
 
         投稿にいいねします
@@ -2498,7 +2547,7 @@ class Client(API):
         """
         return remove_group_highlight_post(self, group_id, post_id, access_token)
 
-    def remove_posts(self, post_ids: List[int], access_token: str = None) -> dict:
+    def remove_posts(self, post_ids: list[int], access_token: str = None) -> dict:
         """
 
         複数の投稿を削除します
@@ -2593,7 +2642,7 @@ class Client(API):
         return create_review(self, user_id, comment, access_token)
 
     def create_reviews(
-        self, user_ids: List[int], comment: str, access_token: str = None
+        self, user_ids: list[int], comment: str, access_token: str = None
     ) -> dict:
         """
 
@@ -2602,7 +2651,7 @@ class Client(API):
         """
         return create_reviews(self, user_ids, comment, access_token)
 
-    def delete_reviews(self, review_ids: List[int], access_token: str = None) -> dict:
+    def delete_reviews(self, review_ids: list[int], access_token: str = None) -> dict:
         """
 
         レターを削除します
@@ -2717,7 +2766,7 @@ class Client(API):
         return get_group_thread_list(self, group_id, from_str, access_token, **params)
 
     def get_thread_joined_statuses(
-        self, ids: List[int], access_token: str = None
+        self, ids: list[int], access_token: str = None
     ) -> dict:
         """
 
@@ -2803,6 +2852,9 @@ class Client(API):
         アカウントを削除します
 
         """
+        answer = input("Are you sure you want to delete your account? Y/N")
+        if answer.lower() != "y":
+            return
         return destroy_user(self, access_token)
 
     def follow_user(self, user_id: int, access_token: str = None) -> dict:
@@ -2813,7 +2865,7 @@ class Client(API):
         """
         return follow_user(self, user_id, access_token)
 
-    def follow_users(self, user_ids: List[int], access_token: str = None) -> dict:
+    def follow_users(self, user_ids: list[int], access_token: str = None) -> dict:
         """
 
         複数のユーザーをフォローします
@@ -2849,7 +2901,7 @@ class Client(API):
 
             - from_timestamp: int = None,
             - number: int = None,
-            - sources: List[str] = None
+            - sources: list[str] = None
 
         """
         return get_follow_recommendations(self, access_token, **params)
@@ -2882,7 +2934,7 @@ class Client(API):
         """
         return get_following_users_born(self, birthdate, access_token)
 
-    def get_footprints(self, access_token: str = None, **params) -> List[Footprint]:
+    def get_footprints(self, access_token: str = None, **params) -> list[Footprint]:
         """
 
         足跡を取得します
@@ -2905,7 +2957,7 @@ class Client(API):
         """
         return get_fresh_user(self, user_id, access_token)
 
-    def get_hima_users(self, access_token: str = None, **params) -> List[UserWrapper]:
+    def get_hima_users(self, access_token: str = None, **params) -> list[UserWrapper]:
         """
 
         暇なユーザーを取得します
@@ -3050,7 +3102,7 @@ class Client(API):
         """
         return get_user_without_leaving_footprint(self, user_id, access_token)
 
-    def get_users(self, user_ids: List[int], access_token: str = None) -> UsersResponse:
+    def get_users(self, user_ids: list[int], access_token: str = None) -> UsersResponse:
         """
 
         複数のユーザーの情報を取得します
@@ -3079,7 +3131,6 @@ class Client(API):
         prefecture: str = None,
         profile_icon_filename: str = None,
         cover_image_filename: str = None,
-        # @Nullable @Part("sns_info") SignUpSnsInfoRequest signUpSnsInfoRequest,
         en: int = None,
         vn: int = None,
     ):
@@ -3231,35 +3282,26 @@ class Client(API):
         """
         return unfollow_user(self, user_id, access_token)
 
-    def update_user(
-        self,
-        nickname: str,
-        biography: str = None,
-        prefecture: str = None,
-        gender: int = None,
-        country_code: str = None,
-        profile_icon_filename: str = None,
-        cover_image_filename: str = None,
-        username: str = None,
-        access_token: str = None,
-    ) -> dict:
+    def update_user(self, nickname: str, access_token: str = None, **params) -> dict:
         """
 
         プロフィールを更新します
 
+        Parameters
+        ----------
+
+            - nickname: str = (required)
+            - biography: str = (optional)
+            - prefecture: str = (optional)
+            - gender: int = (optional)
+            - country_code: str = (optional)
+            - profile_icon_filename: str = (optional)
+            - cover_image_filename: str = (optional)
+            - username: str = (optional)
+            - access_token: str = (optional)
+
         """
-        return update_user(
-            self,
-            nickname,
-            biography,
-            prefecture,
-            gender,
-            country_code,
-            profile_icon_filename,
-            cover_image_filename,
-            username,
-            access_token,
-        )
+        return update_user(self, nickname, access_token, **params)
 
     def block_user(self, user_id: int, access_token: str = None) -> dict:
         """
@@ -3296,7 +3338,7 @@ class Client(API):
         return unblock_user(self, user_id, access_token)
 
     def get_hidden_users_list(
-        self, access_token: str = None, **params: Union[str, int]
+        self, access_token: str = None, **params
     ) -> HiddenResponse:
         """
 
@@ -3319,7 +3361,7 @@ class Client(API):
         """
         return hide_user(self, user_id, access_token)
 
-    def unhide_users(self, user_ids: List[int], access_token: str = None) -> dict:
+    def unhide_users(self, user_ids: list[int], access_token: str = None) -> dict:
         """
 
         ユーザーの非表示を解除します
