@@ -1,7 +1,7 @@
 """
 MIT License
 
-Copyright (c) 2023-present Qvco, Konn
+Copyright (c) 2023-present qvco
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -23,13 +23,14 @@ SOFTWARE.
 """
 
 from enum import Enum
+from .models import Device
 
 
 class Configs:
-    YAYLIB_VERSION = "1.1.2"
-    YAY_API_VERSION = "3.21"
-    YAY_VERSION_NAME = "3.21.0"
-    YAY_API_VERSION_KEY = "c687e24f6a454896891acd68868c7979"
+    YAYLIB_VERSION = "1.2.2"
+    YAY_API_VERSION = "3.24"
+    YAY_VERSION_NAME = "3.24.0"
+    YAY_API_VERSION_KEY = "a9551e9986e04937b68dfd4d9e1343e0"
     YAY_API_KEY = "ccd59ee269c01511ba763467045c115779fcae3050238a252f1bd1a4b65cfec6"
     YAY_SHARED_KEY = "yayZ1"
     YAY_STORE_KEY = "yayZ1payment"
@@ -44,15 +45,24 @@ class Configs:
     YAY_PRODUCTION_HOST = "api.yay.space"
     ID_CARD_CHECK_HOST_PRODUCTION = "idcardcheck.com"
     ID_CARD_CHECK_HOST_STAGING = "stg.idcardcheck.com"
-    COOKIE_PROPERTIES = ["access_token", "refresh_token", "user_id", "email"]
-    USER_AGENT = "android 11 (3.5x 1440x2960 Galaxy S9)"
+    DEVICE = Device(
+        device_type="android",
+        os_version="11",
+        screen_density="3.5",
+        screen_size="1440x2960",
+        model="Galaxy S9",
+    )
+    USER_AGENT = f"{DEVICE.device_type} {DEVICE.os_version} ({DEVICE.screen_density}x {DEVICE.screen_size} {DEVICE.model})"
     REQUEST_HEADERS = {
         "Host": YAY_PRODUCTION_HOST,
-        "X-App-Version": YAY_API_VERSION,
         "User-Agent": USER_AGENT,
-        "X-Device-Info": f"yay {YAY_VERSION_NAME} {USER_AGENT}",
-        "X-Device-Uuid": "",
+        "X-Timestamp": "",
+        "X-App-Version": YAY_API_VERSION,
+        "X-Device-Info": "yay " + YAY_VERSION_NAME + " " + USER_AGENT,
+        "X-Device-UUID": "",
+        "X-Client-IP": "",
         "X-Connection-Type": "wifi",
+        "X-Connection-Speed": "0",
         "Accept-Language": "ja",
         "Content-Type": "application/json;charset=UTF-8",
     }
