@@ -1,7 +1,7 @@
 """
 MIT License
 
-Copyright (c) 2023-present Qvco, Konn
+Copyright (c) 2023-present qvco
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -225,16 +225,6 @@ class ApplicationConfigSettings:
 
 
 class Attachment:
-    __slots__ = (
-        "file",
-        "filename",
-        "original_file_name",
-        "original_file_extension",
-        "natural_width",
-        "natural_height",
-        "is_thumb",
-    )
-
     def __init__(
         self,
         file,
@@ -319,7 +309,8 @@ class ChatRoom:
         "is_group",
         "owner",
         "is_request",
-        "user_setting",
+        "is_pinned",
+        "is_notification_on",
     )
 
     def __init__(self, data):
@@ -346,10 +337,8 @@ class ChatRoom:
             self.owner = User(self.owner)
 
         self.is_request = data.get("is_request")
-
-        self.user_setting = data.get("user_setting")
-        # if self.user_setting is not None:
-        #     self.user_setting = UserSetting(self.user_setting)
+        self.is_pinned = data.get("is_pinned")
+        self.is_notification_on = data.get("is_notification_on")
 
     def __repr__(self):
         return f"ChatRoom(data={self.data})"
@@ -510,7 +499,7 @@ class ConferenceCall:
         "max_participants",
         "conference_call_users",
         "bump_params",
-        "conference_call_users_role",
+        "conference_call_user_roles",
     )
 
     def __init__(self, data):
@@ -547,11 +536,11 @@ class ConferenceCall:
         # if self.bump_params is not None:
         #     self.bump_params = BumpParams(self.bump_params)
 
-        self.conference_call_users_role = data.get("conference_call_usersRole")
-        if self.conference_call_users_role is not None:
-            self.conference_call_users_role = [
+        self.conference_call_user_roles = data.get("conference_call_user_roles")
+        if self.conference_call_user_roles is not None:
+            self.conference_call_user_roles = [
                 ConferenceCallUserRole(conference_call_user_role)
-                for conference_call_user_role in self.conference_call_users_role
+                for conference_call_user_role in self.conference_call_user_roles
             ]
 
     def __repr__(self):
@@ -594,6 +583,25 @@ class CreateGroupQuota:
         return f"CreateGroupQuota(data={self.data})"
 
 
+class Device:
+    def __init__(
+        self,
+        device_type: str,
+        os_version: str,
+        screen_density: str,
+        screen_size: str,
+        model: str,
+    ):
+        self.device_type = device_type
+        self.os_version = os_version
+        self.screen_density = screen_density
+        self.screen_size = screen_size
+        self.model = model
+
+    def __repr__(self):
+        return f"Device(device_type={self.device_type})"
+
+
 class TimelineSettings:
     __slots__ = (
         "data",
@@ -979,18 +987,23 @@ class Message:
         "data",
         "attachment",
         "attachment_android",
+        "attachment_read_count",
         "attachment_thumbnail",
         "conference_call",
+        "parent",
         "created_at",
         "font_size",
         "gif",
         "id",
         "message_type",
         "reactions_count",
+        "reacted",
         "room_id",
         "sticker",
+        "is_sent",
+        "refresh_retry_count",
+        "is_error",
         "text",
-        "user",
         "user_id",
         "video_processed",
         "video_thumbnail_big_url",
@@ -1002,12 +1015,77 @@ class Message:
         self.data = data
         self.attachment = data.get("attachment")
         self.attachment_android = data.get("attachment_android")
+        self.attachment_read_count = data.get("attachment_read_count")
         self.attachment_thumbnail = data.get("attachment_thumbnail")
 
         self.conference_call = data.get("conference_call")
         if self.conference_call is not None:
             self.conference_call = ConferenceCall(self.conference_call)
 
+        self.parent = data.get("parent")
+        if self.parent is not None:
+            self.parent = ParentMessage(self.parent)
+
+        self.created_at = data.get("created_at")
+        self.font_size = data.get("font_size")
+
+        self.gif = data.get("gif")
+        if self.gif is not None:
+            self.gif = GifImage(self.gif)
+
+        self.id = data.get("id")
+        self.message_type = data.get("message_type")
+        self.reactions_count = data.get("reactions_count")
+        self.reacted = data.get("reacted")
+        self.room_id = data.get("room_id")
+
+        self.sticker = data.get("sticker")
+        if self.sticker is not None:
+            self.sticker = Sticker(self.sticker)
+
+        self.is_sent = data.get("is_sent")
+        self.refresh_retry_count = data.get("refresh_retry_count")
+        self.is_error = data.get("is_error")
+        self.text = data.get("text")
+        self.user_id = data.get("user_id")
+        self.video_processed = data.get("video_processed")
+        self.video_thumbnail_big_url = data.get("video_thumbnail_big_url")
+        self.video_thumbnail_url = data.get("video_thumbnail_url")
+        self.video_url = data.get("video_url")
+
+    def __repr__(self):
+        return f"Message(data={self.data})"
+
+
+class ParentMessage:
+    __slots__ = (
+        "data",
+        "attachment",
+        "attachment_android",
+        "attachment_thumbnail",
+        "created_at",
+        "font_size",
+        "gif",
+        "id",
+        "message_type",
+        "reactions_count",
+        "room_id",
+        "sticker",
+        "text",
+        "user_id",
+        "video_processed",
+        "video_thumbnail_url",
+        "video_thumbnail_big_url",
+        "video_url",
+        "parent_id",
+        "reacted",
+    )
+
+    def __init__(self, data):
+        self.data = data
+        self.attachment = data.get("attachment")
+        self.attachment_android = data.get("attachment_android")
+        self.attachment_thumbnail = data.get("attachment_thumbnail")
         self.created_at = data.get("created_at")
         self.font_size = data.get("font_size")
 
@@ -1025,15 +1103,16 @@ class Message:
             self.sticker = Sticker(self.sticker)
 
         self.text = data.get("text")
-        self.user = data.get("user")
         self.user_id = data.get("user_id")
         self.video_processed = data.get("video_processed")
-        self.video_thumbnail_big_url = data.get("video_thumbnail_big_url")
         self.video_thumbnail_url = data.get("video_thumbnail_url")
+        self.video_thumbnail_big_url = data.get("video_thumbnail_big_url")
         self.video_url = data.get("video_url")
+        self.parent_id = data.get("parent_id")
+        self.reacted = data.get("reacted")
 
     def __repr__(self):
-        return f"Message(data={self.data})"
+        return f"ParentMessage(data={self.data})"
 
 
 class MessageTag:
